from Crypto.Cipher import AES
import base64
import hashlib
import datetime
import urllib
import pytz
import array
import operator
import simplejson as json

def generate_sso_token(subdomain_name, sso_key, user_attributes):
    current_time = (datetime.datetime.now(pytz.utc) + datetime.timedelta(minutes=5)).strftime('%Y-%m-%d %H:%M:%S')
    user_attributes.setdefault('expires', current_time)
    user_json = json.dumps(user_attributes, separators=(',',':'))
    iv = "OpenSSL for Ruby"
    block_size = 16

    salted = sso_key + subdomain_name
    saltedHash = hashlib.sha1(salted).digest()[:16]

    json_bytes = array.array('b', user_json[0 : len(user_json)])
    iv_bytes = array.array('b', iv[0 : len(iv)])

    for i in range(0, 16):
        json_bytes[i] = operator.xor(json_bytes[i], iv_bytes[i])

    pad = block_size - len(json_bytes.tostring()) % block_size
    data = json_bytes.tostring() + pad * chr(pad)
    aes = AES.new(saltedHash, AES.MODE_CBC, iv)
    encrypted_bytes = aes.encrypt(data)

    return urllib.quote(base64.b64encode(encrypted_bytes))




import yaml
import unittest
import uservoice

class TestClient(unittest.TestCase):
    def setUp(self):
        super(TestClient, self).setUp()
        with open('test/config.yml') as f:
            self.config = yaml.load(f)
        self.client = uservoice.Client(self.config['subdomain_name'],
                                       self.config['api_key'],
                                       self.config['api_secret'])
    def test_getting_first_10_users(self):
        users = self.client.get("/api/v1/users")['users']
        self.assertEqual(len(users), 10)

    def test_login_as_owner(self):
        owner_token = self.client.login_as_owner()
        user = owner_token.get("/api/v1/users/current")['user']
        self.assertTrue(user['roles']['owner'])

    def test_login_as_regular_user(self):
        token = self.client.login_as('someguy@example.com')
        user = token.get("/api/v1/users/current.json")['user']
        self.assertFalse(user['roles']['owner'])
        self.assertEqual(user['email'], 'someguy@example.com')

    def test_login_with_access_token(self):
        old_client = self.client.login_as('someguy@example.com')

        new_client = uservoice.Client(self.config['subdomain_name'],
                                          self.config['api_key'],
                                          self.config['api_secret'])
        reloaded_token = new_client.login_with_access_token(old_client.token, old_client.secret)

        user = reloaded_token.get("/api/v1/users/current.json")['user']
        self.assertFalse(user['roles']['owner'])
        self.assertEqual(user['email'], 'someguy@example.com')

    def test_should_not_create_KB_article_as_regular_user(self):
        nobody = self.client.login_as('regular_user@example.com')

        def func():
            result = nobody.post("/api/v1/articles.json", {
                'article': { 'title': 'good morning' }
            })

        self.assertRaises(uservoice.Unauthorized, func)

    def test_should_not_get_current_user_as_nobody(self):
        def func():
            result = self.client.get("/api/v1/users/current")

        self.assertRaises(uservoice.Unauthorized, func)

    def test_should_get_access_token_as_owner(self):
        with self.client.login_as_owner() as owner:
          self.assertTrue(owner.get("/api/v1/users/current.json")['user']['roles']['owner'])
          with owner.login_as('regular@example.com') as regular:
              self.assertTrue(owner.get("/api/v1/users/current.json")['user']['roles']['owner'])
              self.user = regular.get("/api/v1/users/current.json")['user']
              self.assertFalse(self.user['roles']['owner'])
        self.assertEqual(self.user['email'], 'regular@example.com')


    def test_should_generate_not_found_on_unexistant_endpoint(self):
        def func():
            result = self.client.get("/api/v1/lanterns/1")

        self.assertRaises(uservoice.NotFound, func)

    def test_should_not_get_unexistant_user(self):
        def func():
            self.client.get("/api/v1/users/86723562378523")

        self.assertRaises(uservoice.NotFound, func)

    def test_should_get_reasonable_collection_using_collection_class(self):
        suggestions = self.client.get_collection("/api/v1/suggestions")
        count = 0
        for suggestion in suggestions:
            count += 1
        self.assertEqual(count, len(suggestions))

    def test_should_get_reasonable_collection_using_collection_class(self):
        suggestions = self.client.get_collection("/api/v1/suggestions", limit=1)
        count = 0
        for suggestion in suggestions:
            count += 1
        self.assertEqual(len(suggestions), 1)

    def test_should_get_page_of_tickets(self):
        tickets = self.client.get("/api/v1/tickets?per_page=2")
        self.assertEqual(len(tickets), 2)



import os
import sys
import re

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(name='uservoice',
      version='v0.0.7',
      description='UserVoice Python library',
      url = 'http://pypi.python.org/pypi/uservoice/',
      long_description=re.sub(r'```[^\s]*', '', open('README.md').read()),
      author='Raimo Tuisku',
      author_email='dev@uservoice.com',
      packages=['uservoice'],
      install_requires=[
          'simplejson',
          'pycrypto',
          'pytz',
          'PyYAML',
          'requests-oauth'],
      test_suite='test',
)


PER_PAGE = 500
class Collection:
    def __init__(self, client, query, limit=2**60):
        self.client = client
        self.query = query
        self.limit = limit
        self.per_page = min(self.limit, PER_PAGE)
        self.pages = {}
        self.response_data = None

    def __len__(self):
        if not self.response_data:
            try:
                self[0]
            except IndexError:
                pass
        return min(self.response_data['total_records'], self.limit)

    def __getitem__(self, i):
        if i == 0 or (i > 0 and i < len(self)):
            return self.load_page(int(i/float(PER_PAGE)) + 1)[i % PER_PAGE]
        else:
            raise IndexError

    def __iter__(self):
        for index in range(len(self)):
            yield self[index]

    def load_page(self, i):
        if not i in self.pages:
            url = self.query
            if '?' in self.query:
                url += '&'
            else:
                url += '?'
            result = self.client.get(url + "per_page=" + str(self.per_page) + "&page=" + str(i))

            if 'response_data' in result:
                self.response_data = result.pop('response_data')
                if len(result.values()) > 0:
                    self.pages[i] = result.values()[0]
            else:
                raise uservoice.NotFound.new('The resource you requested is not a collection')
        return self.pages[i]

